#include "fractroam.h"
#include "png.h"

/* write a png file */
void write_png(char *file_name, png_byte *image)
{
   FILE *fp;
   png_structp png_ptr;
   png_infop info_ptr;
   png_uint_32 k, height=DEFAULT_FRACT_HEIGHT, width=DEFAULT_FRACT_WIDTH;
   png_bytep row_pointers[height];
   png_color sig_bit;

   /* open the file */
   fp = fopen(file_name, "wb");
   if (fp == NULL)
      return;

   png_ptr = png_create_write_struct(PNG_LIBPNG_VER_STRING, NULL, NULL, NULL);

   if (png_ptr == NULL)
   {
      fclose(fp);
      return;
   }

   /* Allocate/initialize the image information data.  REQUIRED */
   info_ptr = png_create_info_struct(png_ptr);
   if (info_ptr == NULL)
   {
      fclose(fp);
      png_destroy_write_struct(&png_ptr,  (png_infopp)NULL);
      return;
   }

   /* Set error handling.  REQUIRED if you aren't supplying your own
    * error hadnling functions in the png_create_write_struct() call.
    */
   if (setjmp(png_ptr->jmpbuf))
   {
      /* If we get here, we had a problem reading the file */
      fclose(fp);
      png_destroy_write_struct(&png_ptr,  (png_infopp)NULL);
      return;
   }

   png_init_io(png_ptr, fp);

   /* Set the image information here.  Width and height are up to 2^31,
    * bit_depth is one of 1, 2, 4, 8, or 16, but valid values also depend on
    * the color_type selected. color_type is one of PNG_COLOR_TYPE_GRAY,
    * PNG_COLOR_TYPE_GRAY_ALPHA, PNG_COLOR_TYPE_PALETTE, PNG_COLOR_TYPE_RGB,
    * or PNG_COLOR_TYPE_RGB_ALPHA.  interlace is either PNG_INTERLACE_NONE or
    * PNG_INTERLACE_ADAM7, and the compression_type and filter_type MUST
    * currently be PNG_COMPRESSION_TYPE_BASE and PNG_FILTER_TYPE_BASE. REQUIRED
    */
   //png_set_IHDR(png_ptr, info_ptr, width, height, bit_depth, PNG_COLOR_TYPE_, PNG_INTERLACE_, PNG_COMPRESSION_TYPE_BASE, PNG_FILTER_TYPE_BASE);
   png_set_IHDR(png_ptr, info_ptr, width, height, 8, PNG_COLOR_TYPE_RGB, PNG_INTERLACE_NONE, PNG_COMPRESSION_TYPE_BASE, PNG_FILTER_TYPE_BASE);

   /* optional significant bit chunk */
   /* if we are dealing with a grayscale image then */
   /* otherwise, if we are dealing with a color image then */
   //sig_bit.red = 8;
   //sig_bit.green = 8;
   //sig_bit.blue = 8;
   //sig_bit.red = true_red_bit_depth;
   //sig_bit.green = true_green_bit_depth;
   //sig_bit.blue = true_blue_bit_depth;
   //png_set_sBIT(png_ptr, info_ptr, sig_bit);

   /* Optionally write comments into the image */
   //text_ptr[0].key = "Title";
   //text_ptr[0].text = "Mona Lisa";
   //text_ptr[0].compression = PNG_TEXT_COMPRESSION_NONE;
   //text_ptr[1].key = "Author";
   //text_ptr[1].text = "Leonardo DaVinci";
   //text_ptr[1].compression = PNG_TEXT_COMPRESSION_NONE;
   //text_ptr[2].key = "Description";
   //text_ptr[2].text = "<long text>";
   //text_ptr[2].compression = PNG_TEXT_COMPRESSION_zTXt;
   //png_set_text(png_ptr, info_ptr, text_ptr, 3);

   /* Write the file header information.  REQUIRED */
   png_write_info(png_ptr, info_ptr);

   /* The easiest way to write the image (you may have a different memory
    * layout, however, so choose what fits your needs best).  You need to
    * use the first method if you aren't handling interlacing yourself.
    */
   for (k = 0; k < height; k++) {
     //row_pointers[k] = (png_byte *) image;
     row_pointers[k] = (png_byte *) (image + k*width*3);
     //row_pointers[k] = (png_byte *) (image + k*width);
   }

   png_write_image(png_ptr, row_pointers);

   png_write_end(png_ptr, info_ptr);

   /* if you allocated any text comments, free them here */
   //free(text_ptr[0]->key);
   //free(text_ptr[0]->text);
   //free(text_ptr[1]->key);
   //free(text_ptr[1]->text);
   //free(text_ptr[2]->key);
   //free(text_ptr[2]->text);

   /* clean up after the write, and free any memory allocated */
   png_destroy_write_struct(&png_ptr, (png_infopp)NULL);

   /* close the file */
   fclose(fp);

   /* that's it */
   return;
}

int main(char* argv, char** argc) {
  int i=1,x,y;
  char filename[80];
  short* julia;
  int test;
  int blank;
  png_byte *image=malloc(sizeof(png_byte)*DEFAULT_FRACT_WIDTH*DEFAULT_FRACT_HEIGHT*3);
  png_byte *imagetemp;
  fract_init();
  do {
    fract_plot();
    fract_update();
    fract_julia_plot();
    julia=fract_julia_get();
    sprintf(filename,"frame%.6d.png",i);
    imagetemp=image;
    blank=1;
    test = *julia + *(julia+1) + *(julia+2);
    for (y=0;y<DEFAULT_FRACT_HEIGHT;y++)
      for (x=0;x<DEFAULT_FRACT_WIDTH;x++) {
        if (blank) if (test != (*julia + *(julia+1) + *(julia+2))) blank=0;
        *(imagetemp++)=(png_byte)*(julia++);
        *(imagetemp++)=(png_byte)*(julia++);
        *(imagetemp++)=(png_byte)*(julia++);
      }
    if (!blank) {
      write_png(filename,image);
      i++;
    }
  } while (1);
  return 0;
}
